function linear_regression(poly, train_method, ss, st)
	%two optional arguments, stepsize (ss) and stopping threshold (st), both real numbers

	%for polynomial fitting
	function A = vandermonde(x, n)
		n = n+1;
		A = ones(size(x, 1), 1);
		for i = 2:n
		  A = [A A(:,i-1).*x];
		end
	end

	%read and prepare data
	X = load('train_x.txt');
	y = load('train_y.txt');
	devX = load('dev_x.txt');
	devy = load('dev_y.txt');
	n = size(X,1);
	if poly > 1
		X = vandermonde(X,poly);
		devX = vandermonde(devX, poly);
	else
		X = [ones(n, 1) X];
		devX = [ones(size(devX, 1), 1) devX];
	end


	%training
	if strcmp('a', train_method)
		%analytic solution
		B = inv(X'*X)*X'*y;
	elseif strcmp('g', train_method)
		%gradient descent solution
		oldloss = realmax;
		B = zeros(size(X,2), 1);
		rr = realmax;
		f = fopen('obj.txt', 'w');
		d = fopen('eval.txt', 'w');
		while rr > st
			B = B - 2/n*ss*X'*(X*B - y);
			newloss = (1/n)*sum((X*B - y).^2);
			rr = (oldloss - newloss)/oldloss;
			oldloss = newloss;
			fprintf(f, '%.3f\n', newloss);
			predictions = devX*B;
			fprintf(d, '%.3f\n', (1/size(devy,1))*sum((predictions - devy).^2));
		end
	end
end
